# rumi.test_msg_rumi.test_reader
# Test the reader for message-based monitoring
#
# Author: Tianshu Li
# Created: Nov.15 2021

"""
Test the reader for message-based monitoring
"""

##########################################################################
# Imports
##########################################################################


import git
import time

from datetime import datetime, timezone
from rumi.msg_rumi.reader import MsgReader


##########################################################################
# MsgReader Test Cases
##########################################################################


class TestMsgReader():
    """
    MsgReader Tests
    """
    def test_fixtures_msg_reader(self, tmpdir):
        """
        Generate fixture repo for testing MsgReader.
        """
        repo_name = "msg_reader_repo"
        repo_path = tmpdir / repo_name
        repo = git.Repo.init(repo_path)
        repo.config_writer().set_value("user", "name", "testrumi").release()
        repo.config_writer().set_value("user", "email", "testrumiemail").release()
        
        initial_file = repo_path / "initial_file.txt"
        initial_file.write_text("", encoding="utf8")
        repo.git.add(A=True)
        repo.git.commit(m="initial commit")

        repo.git.branch("test")
        repo.git.checkout("test")

        content_dir = repo_path / "locales"
        content_dir.mkdir()
        en_dir = content_dir / "en"
        en_dir.mkdir()
        fr_dir = content_dir / "fr"
        fr_dir.mkdir()
        en_file = en_dir / "messages.po"
        fr_file = fr_dir / "messages.po"
        
        en_content = 'Header line.\nmsgid "new msg"\nmsgstr "new msg"'
        fr_content = 'Header line.\nmsgid "new msg"\nmsgstr ""'
        en_file.write_text(en_content, encoding="utf8")
        fr_file.write_text(fr_content, encoding="utf8")
        repo.git.add(A=True)
        repo.git.commit(m='new msg can be added and empty translation is set to ""')
        ts1 = time.time()

        fr_content = 'Header line.\nmsgid "new msg"\nmsgstr "nouveau message"'
        fr_file.write_text(fr_content, encoding="utf8")
        repo.git.add(A=True)
        repo.git.commit(m="new translation is added")
        ts2 = time.time()

        fr_content = 'Header line.\nmsgid "new msg"\nmsgstr ""'
        fr_file.write_text(fr_content, encoding="utf8")
        repo.git.add(A=True)
        repo.git.commit(m='deleted translation is set to ""')
        ts3 = time.time()

        en_content = 'Header line.\n~msgid "new msg"\n~msgstr "new msg"'
        fr_content = 'Header line.\n~msgid "new msg"\n~msgstr ""'
        en_file.write_text(en_content, encoding="utf8")
        fr_file.write_text(fr_content, encoding="utf8")
        repo.git.add(A=True)
        repo.git.commit(m='deleted msg is set to "deleted"')
        ts4 = time.time()
        return str(repo_path), [ts1, ts2, ts3, ts4]

    def test_parse_history(self, tmpdir):
        """
        Assert git history is correctly parsed into a commit dictionary.
        """
        repo_path, ts = self.test_fixtures_msg_reader(tmpdir)
        # Truncate time.time() stamps due to git log timestamp accuracy
        ts = [float(int(timestamp)) for timestamp in ts]
        
        reader = MsgReader(
            content_path="locales/", extensions=["po"], src_lang="en",
            repo_path=repo_path, branch="test"
        )
        got = reader.parse_history()
        want = {
            '"new msg"': {
                'en': {
                    'filename': 'locales/en/messages.po', 
                    'ft': ts[0], 
                    'lt': ts[3], 
                    'history': [(ts[0], '"new msg"'), (ts[3], '"deleted"')]
                }, 
                'fr': {
                    'filename': 'locales/fr/messages.po', 
                    'ft': ts[0], 
                    'lt': ts[3], 
                    'history': [
                        (ts[0], '""'), (ts[1], '"nouveau message"'), 
                        (ts[2], '""'), (ts[3], '"deleted"')
                    ]
                }
            }
        }
        assert got == want

    def test_parse_timestamp(self, tmpdir):
        """
        Assert correct float timestamp is given based on a string in this format:
        "Thu Nov 11 22:45:36 2021 +0800".
        """
        now = datetime.now(timezone.utc)
        s = now.strftime("%a %b %d %H:%M:%S %Y %z")

        reader = MsgReader(
            content_path="locales/", extensions=["po"], src_lang="en"
        )
        ts = reader.parse_timestamp(s)
        
        assert ts == float(int(datetime.timestamp(now)))

    def test_parse_lang(self, tmpdir):
        """
        Assert correct language is parsed from filename.
        """
        filename = "web/src/locales/en/messages.po"
        reader = MsgReader(
            content_path="locales/", extensions=["po"], src_lang="en"
        )
        lang = reader.parse_lang(filename)

        assert lang == "en"